"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateUsage = exports.FishyClient = void 0;
const discord_js_1 = require("discord.js");
const types_1 = require("./types");
const fs = __importStar(require("fs"));
const path_1 = require("path");
const axios_1 = __importDefault(require("axios"));
const ApplicationCommandCompare_1 = require("./utils/ApplicationCommandCompare");
const Embeds_1 = require("./utils/Embeds");
const mongoose_1 = __importDefault(require("mongoose"));
const SilenceCommand = __importStar(require("./commands/Silence"));
const CommandsCommand = __importStar(require("./commands/Commands"));
const ButtonInteraction_1 = __importDefault(require("./structures/ButtonInteraction"));
const SlashCommand_1 = require("./structures/SlashCommand");
const SelectInteraction_1 = __importDefault(require("./structures/SelectInteraction"));
function hasOwnProperty(obj, prop) {
    return obj.hasOwnProperty(prop);
}
// The main client!
class FishyClient extends discord_js_1.Client {
    constructor(options, client_options) {
        super(client_options);
        this.commands = new discord_js_1.Collection();
        this.componentCommands = new discord_js_1.Collection();
        this.categories = new discord_js_1.Collection();
        this.fishy_options = options;
        // Checking for things
        if (!options.token)
            throw Error("You must specify a token");
        if (!options.db_uri && !options.disable_db_connect)
            throw Error("You must specify a database uri");
        if (!options.guild_model && !options.disable_db_connect)
            throw Error("You must specify a mongoose guild model");
        if (!options.guild_model.schema.path("id") && !options.disable_db_connect)
            throw Error("The mongoose model needs the 'id' path as a String");
        if (!options.guild_model.schema.path("settings") && !options.disable_db_connect)
            throw Error("The mongoose model needs the 'settings' path as a Map");
        //if (!options.event_array && !options.event_dir) throw Error("You must specify an event directory or event array");
        if (!options.cmd_array && !options.cmd_dir)
            throw Error("You must specify a command directory or command array");
        this.token = options.token;
        this.GuildModel = options.guild_model;
        //Loading commands and events
        if (!options.disable_load_on_construct)
            this.load();
        process.on("SIGINT", async () => {
            await this.logToChannel("SIGINT signal received: stopping bot");
            console.log("SIGINT signal received: stopping bot");
            await Promise.all([this.destroy(), mongoose_1.default.disconnect()]);
            await new Promise((resolve) => {
                setTimeout(() => {
                    resolve("");
                }, 1000);
            });
            console.log("exiting");
            process.exit();
        });
    }
    // Loads the events from a given event_dir
    async load_events(directory) {
        return new Promise((resolve, reject) => {
            if (directory) {
                fs.readdir(path_1.join(process.cwd(), directory), (err, files) => {
                    if (err)
                        return;
                    files.forEach((file, index, array) => {
                        const event = require(path_1.join(process.cwd(), directory, file));
                        if (Object.keys(discord_js_1.Constants.Events).includes(event.trigger.toUpperCase()) ||
                            Object.values(discord_js_1.Constants.Events)
                                .map((v) => v.toLowerCase())
                                .includes(event.trigger.toLowerCase())) {
                            this.on(event.trigger, event.run.bind(null, this));
                        }
                        else {
                            // @ts-ignore
                            this.ws.on(event.trigger, event.run.bind(null, this));
                        }
                        if (index === array.length - 1)
                            resolve(true);
                    });
                });
            }
            else {
                resolve(false);
            }
        });
    }
    // Loads the commands from a given cmd_dir
    async load_commands(directory) {
        return new Promise(async (resolve, reject) => {
            if (directory) {
                try {
                    const dirs = await fs.promises.readdir(path_1.join(process.cwd(), directory));
                    // Go through all the subdirecoties (categories)
                    dirs.forEach(async (dir, dir_index, dir_array) => {
                        const category_path = path_1.join(process.cwd(), directory, dir);
                        let files = await fs.promises.readdir(category_path);
                        if (!files) {
                            if (dir_index === dir_array.length - 1)
                                resolve(true);
                            return;
                        }
                        // Find a the index file for a category
                        // That file says what the category does and that kind of stuff
                        let index_path = files.find((file) => file.startsWith("index.") && !file.endsWith(".d.ts"));
                        let category = undefined;
                        // Create a anew category if an index file was found
                        if (index_path) {
                            category = require(path_1.join(category_path, index_path));
                            if (category) {
                                category.commands = category.commands ?? [];
                                this.categories.set(category.name, category);
                            }
                        }
                        // Go through all the files in the direcory
                        files.forEach((file, file_index, file_array) => {
                            // Ignore all the non js/ts files, and the index file
                            if ((file.endsWith(".js") || file.endsWith(".ts")) &&
                                !file.startsWith("index.") &&
                                !file.endsWith(".d.ts")) {
                                let command_path = path_1.join(category_path, file);
                                let new_unknown_command = require(command_path);
                                if (hasOwnProperty(new_unknown_command.config, "name")) {
                                    const config = new_unknown_command.config;
                                    const rawCommandCode = new_unknown_command.run;
                                    const new_command = { config: config, run: rawCommandCode };
                                    // If the command doesnt have a given category, make the directory name its category
                                    if (!config.category) {
                                        config.category = dir;
                                    }
                                    // Check if the category already exists, and doesnt include the command already
                                    if (this.categories.has(config.category) &&
                                        !this.categories.get(config.category).commands.includes(config.name)) {
                                        let old_category = this.categories.get(config.category);
                                        old_category.commands.push(config.name);
                                        this.categories.set(config.category, old_category);
                                    }
                                    // Adds the command to the command Collection
                                    this.commands.set(config.name.toLowerCase(), new_command);
                                }
                                else {
                                    const config = new_unknown_command.config;
                                    const rawCommandCode = new_unknown_command.run;
                                    const new_command = { config: config, run: rawCommandCode };
                                    this.componentCommands.set(config.custom_id, new_command);
                                }
                            }
                            // If this was the last file, resolve
                            if (file_index === file_array.length - 1 && dir_index === dir_array.length - 1)
                                resolve(true);
                        });
                    });
                }
                catch (err) {
                    throw err;
                }
            }
            else {
                resolve(false);
            }
        });
    }
    // Loads interactions and updates them with discord if needed
    async load_interactions(force_update, user_id) {
        // Fetch discord user for getting the user id
        if (!user_id) {
            const userdataPath = `https://discord.com/api/v8/users/@me`;
            const userdata = await axios_1.default.get(userdataPath, {
                headers: { Authorization: `Bot ${this.token}` },
            });
            const user = userdata.data;
            user_id = user.id;
        }
        let SlashCommandsUrl;
        if (this.fishy_options.dev_guild_id) {
            SlashCommandsUrl = `https://discord.com/api/v8/applications/${user_id}/guilds/${this.fishy_options.dev_guild_id}/commands`;
        }
        else {
            SlashCommandsUrl = `https://discord.com/api/v8/applications/${user_id}/commands`;
        }
        if (force_update) {
            this.commands.forEach((command) => {
                const interaction = command.config.interaction_options;
                axios_1.default
                    .post(SlashCommandsUrl, interaction, {
                    headers: { Authorization: `Bot ${this.token}` },
                })
                    .catch((err) => {
                    console.log(`\nError:`);
                    console.log(err);
                    console.log(interaction);
                    console.log(err.response);
                });
            });
        }
        else {
            const discordSlashCommandsData = await axios_1.default.get(SlashCommandsUrl, {
                headers: { Authorization: `Bot ${this.token}` },
            });
            const discordSlashCommands = discordSlashCommandsData.data;
            let botSlashCommands = this.commands.map((command) => command.config.interaction_options);
            let discord_done = [];
            botSlashCommands.forEach((fishyBotSlashCommand) => {
                const botSlashCommand = ApplicationCommandCompare_1.ApplicationCommandBuild(fishyBotSlashCommand);
                let discord_command = discordSlashCommands.find((cmd) => cmd.name == botSlashCommand.name);
                if (!discord_command)
                    return axios_1.default
                        .post(SlashCommandsUrl, botSlashCommand, {
                        headers: { Authorization: `Bot ${this.token}` },
                    })
                        .then((res) => console.log(`POST - ${res.status} Interaction: "${botSlashCommand.name}", `))
                        .catch((err) => {
                        console.log(err.response);
                        console.log(err.response.status);
                    });
                discord_done.push(discord_command.id);
                if (!ApplicationCommandCompare_1.ApplicationCommandCompare(botSlashCommand, discord_command))
                    return axios_1.default
                        .patch(SlashCommandsUrl + `/${discord_command.id}`, botSlashCommand, {
                        headers: { Authorization: `Bot ${this.token}` },
                    })
                        .then((res) => console.log(`PATCH - ${res.status} Interaction: "${botSlashCommand.name}", `))
                        .catch((err) => {
                        console.log(err.response);
                        console.log(err.response.status);
                    });
            });
            discordSlashCommands.forEach((cmd) => {
                if (!discord_done.includes(cmd.id)) {
                    axios_1.default
                        .delete(SlashCommandsUrl + `/${cmd.id}`, {
                        headers: { Authorization: `Bot ${this.token}` },
                    })
                        .then((res) => console.log(`DELETE - ${res.status} Interaction: "${cmd.name}", `))
                        .catch((err) => {
                        console.log(err.response);
                        console.log(err.response.status);
                    });
                }
            });
        }
    }
    // Load the interaction command handler
    async load_commandhandler() {
        this.ws.on(
        // @ts-ignore
        "INTERACTION_CREATE", async (raw_interaction) => {
            if (!raw_interaction.data)
                return;
            // Check bot perms
            // TODO: Make it send a message sayin all the perms the user needs
            function check_perms(interaction, perms) {
                let failed = false;
                perms.forEach((perm) => {
                    if (!interaction.member.hasPermission(perm)) {
                        failed = true;
                    }
                });
                return !failed;
            }
            if (raw_interaction.type !== types_1.InteractionType.ApplicationCommand) {
                if (raw_interaction.type !== types_1.InteractionType.MessageComponent)
                    return;
                const custom_id = raw_interaction.data.custom_id;
                const commands = this.componentCommands.filter((cmd) => (cmd.config.atStart && custom_id.startsWith(cmd.config.custom_id)) || cmd.config.custom_id === custom_id);
                let interaction;
                if (raw_interaction.data.component_type == types_1.ComponentType.Button) {
                    const temp = raw_interaction;
                    const raw_button_interaction = temp;
                    interaction = new ButtonInteraction_1.default(this, raw_button_interaction);
                }
                else {
                    const temp = raw_interaction;
                    const raw_select_interaction = temp;
                    interaction = new SelectInteraction_1.default(this, raw_select_interaction);
                }
                for (let [name, command] of commands) {
                    if (command.config.bot_needed && !interaction.guild)
                        return interaction.sendSilent(`To use this button add the bot to a server`);
                    if (command.config.user_perms?.[0] && !check_perms(interaction, command.config.user_perms)) {
                        return interaction.sendSilent(`You do not have the required permissions to use this button.\nPermissions required: \`${command.config.user_perms.join(", ")}\``);
                    }
                    command.run(this, interaction);
                }
            }
            else {
                let interaction = new SlashCommand_1.SlashCommand(this, raw_interaction);
                let command = this.commands.get(interaction.name);
                if (!command) {
                    if (!this.fishy_options.disable_msg_notfound)
                        interaction.sendSilent(`This interaction doesn't seem to exist, if you think this is a mistake, please contact ${this.fishy_options.author}`);
                    return;
                }
                if (command.config.bot_needed === true) {
                    if (!interaction.guild) {
                        return interaction.sendSilent(`To use this command add the bot to a server`);
                    }
                }
                if (command.config.bot_perms?.[0]) {
                    let failed = false;
                    command.config.bot_perms.forEach((perm) => {
                        if (!interaction.guild.me?.hasPermission(perm)) {
                            failed = true;
                        }
                    });
                    if (failed) {
                        return interaction.sendSilent(`The bot doesn't have the required permissions to run this command\nPermissions needed: \`${command.config.bot_perms.join(", ")}\``);
                    }
                }
                if (command.config.user_perms?.[0] && !check_perms(interaction, command.config.user_perms)) {
                    return interaction.sendSilent(`You do not have the required permissions to run this command.\nPermissions required: \`${command.config.user_perms.join(", ")}\``);
                }
                if (command.config.interaction_options.user_perms?.[0] &&
                    !check_perms(interaction, command.config.interaction_options.user_perms)) {
                    return interaction.sendSilent(`You do not have the required permissions to run this command.\nPermissions required: \`${command.config.interaction_options.user_perms.join(", ")}\``);
                }
                if (interaction.data.options?.[0] && command.config.interaction_options.options?.[0]) {
                    function check_option_perms(interaction_options, config_options) {
                        if (!interaction_options?.[0] || !config_options?.[0])
                            return true;
                        for (let option of interaction_options) {
                            const conf_option = config_options.find((opt) => opt.name == option.name);
                            if (conf_option) {
                                if (conf_option.user_perms && !check_perms(interaction, conf_option.user_perms)) {
                                    interaction.sendSilent(`You do not have the required permissions to run this command.\nPermissions required: \`${conf_option.user_perms.join(", ")}\``);
                                    return false;
                                }
                                if (option.options?.[0] && conf_option.options?.[0]) {
                                    return check_option_perms(option.options, conf_option.options);
                                }
                            }
                        }
                        return true;
                    }
                    if (!check_option_perms(interaction.data.options, command.config.interaction_options.options)) {
                        return;
                    }
                }
                try {
                    await command.run(this, interaction);
                }
                catch (err) {
                    console.error(err);
                    if (!this.fishy_options.disable_msg_error)
                        return;
                    let msg = `An error seems to have occurred in the command \`${interaction.name}\`: \n\`\`\`${err}\`\`\``;
                    let embed = new Embeds_1.ErrorEmbed(`An error seems to have occurred in the command: "${interaction.name}"`, `Reason: \n\`\`\`${err}\`\`\``);
                    if (interaction.response_used) {
                        interaction.send_webhook(embed);
                    }
                    else {
                        interaction.sendSilent(msg);
                    }
                }
                finally {
                }
            }
        });
    }
    // Generate a help command
    async help_command() {
        const info_cat = this.categories.get("info");
        if (info_cat) {
            if (info_cat.commands) {
                info_cat.commands.push("help");
            }
            else {
                info_cat.commands = ["help"];
            }
            this.categories.set("info", info_cat);
        }
        let cmd = {
            config: {
                category: "info",
                bot_needed: false,
                name: "help",
                interaction_options: {
                    name: "help",
                    description: "Info about the bot commands and categories",
                    usage: "/help category info | /help command ping",
                    options: [
                        {
                            name: "help",
                            description: "Basic help command",
                            type: types_1.ApplicationCommandOptionType.SUB_COMMAND,
                        },
                        {
                            name: "category",
                            description: "To get help about a category",
                            type: types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP,
                            options: this.categories.map((category) => {
                                let obj = {
                                    name: category.name,
                                    description: category.description,
                                    type: types_1.ApplicationCommandOptionType.SUB_COMMAND,
                                    options: [
                                        {
                                            name: "command",
                                            description: "command to select",
                                            type: types_1.ApplicationCommandOptionType.STRING,
                                            choices: category.commands?.map((command) => {
                                                let choice = {
                                                    name: command,
                                                    value: "cmd_" + command,
                                                };
                                                return choice;
                                            }),
                                        },
                                    ],
                                };
                                return obj;
                            }),
                        },
                        {
                            name: "command",
                            description: "Get help about a specific command",
                            type: types_1.ApplicationCommandOptionType.SUB_COMMAND,
                            options: [
                                {
                                    name: "name",
                                    description: "The command's name",
                                    required: true,
                                    type: types_1.ApplicationCommandOptionType.STRING,
                                },
                            ],
                        },
                    ],
                },
            },
            run: async (client, interaction) => {
                let cmd_help = (input) => {
                    let input_split = input.split(/(?:,|\s+|\.)/gi);
                    let cmd = this.commands.get(input_split.shift() || "");
                    const cmd_name = cmd?.config.name || cmd?.config.interaction_options.name;
                    if (!cmd) {
                        return new Embeds_1.ErrorEmbed(`No command found with the name "${input.split(/(,|\s|\.|:)/gi)[0]}"`);
                    }
                    if (cmd.config.interaction_options.help_embed)
                        return cmd.config.interaction_options.help_embed;
                    let cmd_title = cmd.config.name;
                    let interaction_config = cmd.config.interaction_options;
                    if (input_split[0] && interaction_config) {
                        let current = input_split.shift();
                        while (current) {
                            if (!interaction_config) {
                                return new Embeds_1.ErrorEmbed(`The command \"${cmd.config.name}\" doesn't have any subcommands`);
                            }
                            let option = interaction_config.options?.find((opt) => opt.name.toLowerCase() === current.trim().toLowerCase());
                            if (!option) {
                                return new Embeds_1.ErrorEmbed(`The subcommand \"${current.trim().toLowerCase()}\" doesn't exist on "${cmd.config.name}"`, `You can run \`/help command ${cmd}\``);
                            }
                            else if (option) {
                                interaction_config = option;
                                cmd_title += ` ${option.name}`;
                            }
                            current = input_split.shift();
                        }
                    }
                    let embed = new discord_js_1.MessageEmbed().setAuthor(this.user.tag, this.user.displayAvatarURL(), `https://discord.com/oauth2/authorize?client_id=${this.user.id}&permissions=8&scope=bot%20applications.commands`);
                    if (interaction_config.color)
                        embed.setColor(interaction_config.color);
                    else
                        embed.setColor(this.categories.get(cmd.config.category || "")?.help_embed_color || "RANDOM");
                    if (interaction_config.title)
                        embed.setTitle(interaction_config.title);
                    else
                        embed.setTitle(`'/${cmd_title}' - Command Help`);
                    let desc = `${interaction_config.description}\n 
Usage: \`${interaction_config.usage || `/${generateUsage(cmd_title, interaction_config, true)}` || "no specific usage"}\`
User required perms: \`${interaction_config.user_perms?.join(", ") || "None"}\`
Bot user needed: \`${cmd.config.bot_needed}\`
`;
                    const subcommands = [];
                    if (interaction_config.options?.find((opt) => opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND ||
                        opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP)) {
                        interaction_config.options?.forEach((opt) => {
                            if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                subcommands.push(`${cmd_title} ${opt.name}`);
                            }
                            else if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP && opt.options?.[0]) {
                                opt.options.forEach((opt2) => {
                                    if (opt2.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                        subcommands.push(`${cmd_title} ${opt.name} ${opt2.name}`);
                                    }
                                });
                            }
                        });
                        desc += `Sub-Commands: \n\`${subcommands?.[0] ? subcommands.join("`,\n`") : "None"}\``;
                    }
                    embed.setDescription(desc);
                    embed.setFooter(`bot perms: ${cmd.config.bot_perms?.join(", ") || "None"} `);
                    return embed;
                };
                let cat_help = (cat_name) => {
                    if (!cat_name)
                        return new Embeds_1.ErrorEmbed(`Category "${cat_name}" not found`);
                    let cat = this.categories.get(cat_name);
                    if (!cat) {
                        cat = this.categories.get(cat_name.toLowerCase());
                        if (!cat) {
                            cat = this.categories.get(this.categories.keyArray()[this.categories
                                .keyArray()
                                .map((key) => key.toLowerCase())
                                .indexOf(cat_name.toLowerCase())]);
                            if (!cat) {
                                return new Embeds_1.ErrorEmbed(`No category found for: "${cat_name}"`);
                            }
                        }
                    }
                    if (cat.help_embed)
                        return cat.help_embed;
                    let embed = new discord_js_1.MessageEmbed().setAuthor(this.user.tag, this.user.displayAvatarURL(), `https://discord.com/oauth2/authorize?client_id=${this.user.id}&permissions=8&scope=bot%20applications.commands`);
                    if (cat.help_embed_color)
                        embed.setColor(cat.help_embed_color);
                    else
                        embed.setColor("RANDOM");
                    if (cat.help_embed_title)
                        embed.setTitle(cat.help_embed_title);
                    else
                        embed.setTitle(`${cat.name} - Category Help`);
                    embed.setDescription(`${cat.description}
${cat.commands
                        ?.map((command_name) => {
                        let cmd = this.commands.get(command_name);
                        if (cmd) {
                            const interaction = cmd.config.interaction_options;
                            if (interaction.options?.find((opt) => opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND ||
                                opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP)) {
                                let strings = [];
                                interaction.options.forEach((opt) => {
                                    if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                        strings.push(`**${command_name} ${opt.name}** \`${opt.description}\``);
                                    }
                                    else if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP && opt.options?.[0]) {
                                        opt.options.forEach((opt2) => {
                                            if (opt2.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                                strings.push(`**${command_name} ${opt.name} ${opt2.name}** \`${opt2.description}\``);
                                            }
                                        });
                                    }
                                });
                                return strings.join("\n");
                            }
                            else {
                                let description = cmd?.config.interaction_options.description;
                                return `**${command_name}** \`${description}\``;
                            }
                        }
                        return `**${command_name}**`;
                    })
                        ?.join("\n")}
`);
                    return embed;
                };
                if (interaction.data.options.find((arg) => arg.name == "category")) {
                    let cat_arg = interaction.data.options.find((arg) => arg.name == "category");
                    if (cat_arg?.options?.[0]) {
                        if (typeof cat_arg.options[0].options?.[0]?.value === "string" &&
                            cat_arg.options[0].options?.[0].value?.startsWith("cmd_")) {
                            let cmd_name = cat_arg.options[0].options[0].value.slice(4);
                            let cmd = this.commands.get(cmd_name);
                            if (!cmd?.config?.name) {
                                return interaction.send(new Embeds_1.ErrorEmbed(`Command "${cmd_name}" not found`));
                            }
                            interaction.send(cmd_help(cmd_name));
                        }
                        else {
                            let cat_name = cat_arg.options[0].name;
                            interaction.send(cat_help(cat_name));
                        }
                    }
                }
                else if (interaction.data.options.find((arg) => arg.name == "command")?.options?.[0]?.name === "name") {
                    let cmd_name = interaction.data.options.find((arg) => arg.name == "command").options[0].value;
                    if (!cmd_name || typeof cmd_name !== "string") {
                        return interaction.send(new Embeds_1.ErrorEmbed(`Command "${cmd_name}" not found`));
                    }
                    interaction.send(cmd_help(cmd_name));
                }
                else {
                    const embed = new discord_js_1.MessageEmbed().setTitle("Help page").setDescription(`**Current categories:**
          ${this.categories
                        .map((val, key) => `**${val.name}**\nDesc: \`${val.description}\`\nCommands: ${val.commands
                        ?.map((command_name) => {
                        let command = this.commands.get(command_name);
                        const interaction = command?.config.interaction_options;
                        if (!command || !interaction)
                            return `\`${command_name}\``;
                        else if (interaction.options?.find((opt) => opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND ||
                            opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP)) {
                            let strings = [];
                            interaction.options.forEach((opt) => {
                                if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                    strings.push(`\`${command_name} ${opt.name}\``);
                                }
                                else if (opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP && opt.options?.[0]) {
                                    opt.options.forEach((opt2) => {
                                        if (opt2.type === types_1.ApplicationCommandOptionType.SUB_COMMAND) {
                                            strings.push(`\`${command_name} ${opt.name} ${opt2.name}\``);
                                        }
                                    });
                                }
                            });
                            return strings.join(", ");
                        }
                        else {
                            return `\`${command_name}\``;
                        }
                    })
                        .join(", ") || "`None`"}`)
                        .join("\n")}
          `);
                    embed.setAuthor(this.user.tag, this.user.displayAvatarURL(), `https://discord.com/oauth2/authorize?client_id=${this.user.id}&permissions=8&scope=bot%20applications.commands`);
                    embed.setColor("RANDOM");
                    embed.setTimestamp();
                    interaction.send(embed);
                }
            },
        };
        return cmd;
    }
    // Load normal commands
    load_silence_command() {
        this.commands.set(SilenceCommand.config.name, SilenceCommand);
        const info_cat = this.categories.get("info");
        if (info_cat) {
            if (info_cat.commands) {
                info_cat.commands.push("silence");
            }
            else {
                info_cat.commands = ["silence"];
            }
            this.categories.set("info", info_cat);
        }
    }
    load_commands_command() {
        this.commands.set(CommandsCommand.config.name, CommandsCommand);
        const info_cat = this.categories.get("info");
        if (info_cat) {
            if (info_cat.commands) {
                info_cat.commands.push("commands");
            }
            else {
                info_cat.commands = ["commands"];
            }
            this.categories.set("info", info_cat);
        }
    }
    // Connect to the mongo db database
    async load_db() {
        try {
            mongoose_1.default.connect(this.fishy_options.db_uri, { useNewUrlParser: true, useUnifiedTopology: true });
        }
        catch (err) {
            console.log(err);
            throw Error("Failed to connect to the MongoDB server: \n" + err);
        }
        finally {
            return;
        }
    }
    // Catch discord errors
    async load_error_handler() {
        this.on("error", (err) => {
            this.logToChannel(new Embeds_1.ErrorEmbed("Discord api error", err.message));
            console.error("DISCORD API ERROR!!!");
            console.log(err);
        });
        this.on("warn", (msg) => {
            this.logToChannel(new Embeds_1.WarnEmbed("Discord api warn", msg));
            console.error("discord api warn");
            console.warn(msg);
        });
        /*this.on("debug", (msg) => {
          //this.logToChannel("Api Debug: `" + msg + "`");
          console.error("discord api debug");
          console.warn(msg);
        });*/
    }
    async load_message_handlers() {
        this.on("ready", () => {
            console.log("READY");
            this.logToChannel("**STARTED BOT!!!!**");
        });
        this.on("disconnect", () => {
            console.log("disconnect");
            this.logToChannel("**DISCONNECTED BOT!!!!**");
        });
        this.on("guildCreate", (guild) => {
            const msg = `guildCreate "${guild.name}"\nOwner: ${guild.owner?.displayName}\nMember count: ${guild.memberCount}\nID: ${guild.id}`;
            this.logToChannel(msg);
            console.log(msg);
        });
        this.on("guildDelete", (guild) => {
            const msg = `guildDelete "${guild.name}"\nOwner: ${guild.owner?.displayName}\nMember count: ${guild.memberCount}\nID: ${guild.id}`;
            this.logToChannel(msg);
            console.log(msg);
        });
    }
    async logToChannel(message) {
        if (this.fishy_options.info_channel_id) {
            const channel = await this.channels.fetch(this.fishy_options.info_channel_id);
            if (channel && channel.isText())
                return channel.send(message);
        }
    }
    // Load command :)
    async load() {
        const options = this.fishy_options;
        if (!options.disable_discord_error_catching)
            await this.load_error_handler();
        if (options.info_channel_id)
            await this.load_message_handlers();
        if (!options.disable_db_connect)
            await this.load_db();
        if (options.event_array) {
            options.event_array.forEach((event) => {
                if (Object.keys(discord_js_1.Constants.Events).includes(event.trigger.toUpperCase()) ||
                    Object.values(discord_js_1.Constants.Events)
                        .map((v) => v.toLowerCase())
                        .includes(event.trigger.toLowerCase())) {
                    this.on(event.trigger, event.run.bind(null, this));
                }
                else {
                    // @ts-ignore
                    this.ws.on(event.trigger, event.run.bind(null, this));
                }
            });
        }
        if (options.cmd_array) {
            options.cmd_array.forEach((new_command) => {
                new_command.config.category = new_command.config.category ?? "debug";
                if (this.categories.has(new_command.config.category) &&
                    !this.categories.get(new_command.config.category).commands.includes(new_command.config.name)) {
                    let old_category = this.categories.get(new_command.config.category);
                    old_category.commands.push(new_command.config.name);
                    this.categories.set(new_command.config.category, old_category);
                }
                this.commands.set(new_command.config.name, new_command);
            });
        }
        await Promise.all([this.load_events(options.event_dir), this.load_commands(options.cmd_dir)]);
        if (!options.disable_help_command) {
            let help_cmd = await this.help_command();
            this.commands.set(help_cmd.config.name, help_cmd);
        }
        if (!options.disable_silence_mode)
            this.load_silence_command();
        if (!options.disable_commands_command)
            this.load_commands_command();
        if (!options.disable_interaction_load)
            await this.load_interactions();
        if (!options.disable_command_handler)
            await this.load_commandhandler();
    }
}
exports.FishyClient = FishyClient;
function generateUsage(title, config, types) {
    let str = `${title.trim()} `;
    if (!config.options?.find((opt) => opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND ||
        opt.type === types_1.ApplicationCommandOptionType.SUB_COMMAND_GROUP)) {
        if (types) {
            config.options?.forEach((opt) => {
                if (opt.required) {
                    str += `<${opt.name}: ${Object.keys(types_1.ApplicationCommandOptionType)[Object.values(types_1.ApplicationCommandOptionType).indexOf(opt.type)].toLowerCase()}> `;
                }
                else {
                    str += `(${opt.name}: ${Object.keys(types_1.ApplicationCommandOptionType)[Object.values(types_1.ApplicationCommandOptionType).indexOf(opt.type)].toLowerCase()}) `;
                }
            });
        }
    }
    else {
        str +=
            "[" +
                config.options
                    .map((opt) => {
                    return `${opt.name}`;
                })
                    .join("|") +
                "]";
    }
    return str;
}
exports.generateUsage = generateUsage;
