"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Interaction = void 0;
const types_1 = require("../types");
const axios_1 = __importDefault(require("axios"));
const DbUtils = __importStar(require("../utils/DbUtils"));
const Silence = __importStar(require("../commands/Silence"));
class Interaction {
    constructor(client, raw_interaction) {
        this.client = client;
        this.raw_interaction = raw_interaction;
        this.type = raw_interaction.type;
        this.id = raw_interaction.id;
        this.token = raw_interaction.token;
        this.guild_id = raw_interaction.guild_id;
        this.channel_id = raw_interaction.channel_id;
        this.raw_member = raw_interaction.member;
        this.raw_user = raw_interaction.user || raw_interaction.member.user;
        this.response_used = false;
    }
    // default interaction response
    async send(message, options) {
        if (Silence.isSilent(this.raw_member?.user?.id || "") && options?.flags !== 64) {
            return this.sendSilent(message, options);
        }
        if (this.response_used) {
            if (options?.flags !== 64) {
                return this.send_webhook(message, options);
            }
            else {
                return this.send_webhook(message, options);
            }
        }
        let embed;
        if (typeof message == "object") {
            embed = message;
            message = undefined;
        }
        let DATA = { content: message, embeds: embed ? [embed] : undefined };
        if (options) {
            DATA = Object.assign(DATA, options);
        }
        this.response_used = true;
        try {
            return await axios_1.default.post(`https://discord.com/api/v9/interactions/${this.id}/${this.token}/callback`, {
                type: types_1.InteractionResponseType.ChannelMessageWithSource,
                data: DATA,
            });
        }
        catch (err) {
            console.error(err);
            throw new Error(err);
        }
    }
    // Send an ephemeral message (only the command caller can see the message)
    async sendSilent(message, options) {
        let DATA = { flags: 64 };
        if (options) {
            DATA = Object.assign(DATA, options);
        }
        return this.send(message, DATA);
    }
    // Ack to edit later
    async defer() {
        this.response_used = true;
        return await axios_1.default.post(`https://discord.com/api/v9/interactions/${this.id}/${this.token}/callback`, {
            type: types_1.InteractionResponseType.DeferredChannelMessageWithSource,
            data: {},
        });
    }
    // Edit the original message, or a different message sent with the interaction token
    async edit(message, options, message_id) {
        if (!options) {
            options = {};
        }
        if (!message_id && options && typeof options == "string") {
            message_id = options;
            options = undefined;
        }
        if (typeof message !== "string" && typeof options !== "string") {
            if (!options) {
                options = { embeds: [message] };
            }
            else if (options.embeds) {
                options.embeds.push(message);
            }
            else {
                options.embeds = [message];
            }
            message = "";
        }
        else if (typeof message == "string" && typeof options !== "string") {
            options.content = message;
        }
        if (!message_id) {
            message_id = "@original";
        }
        return await axios_1.default.patch(`https://discord.com/api/v9/webhooks/${this.client.user.id}/${this.token}/messages/${message_id}`, options);
    }
    // Delete the original message, or a different message sent with the interaction token
    async delete(message_id) {
        if (!message_id) {
            message_id = "@original";
        }
        return await axios_1.default.delete(`https://discord.com/api/v9/webhooks/${this.client.user.id}/${this.token}/messages/${message_id}`);
    }
    // Send a message via the interaction webhook, can only be used after interaction.send()
    async send_webhook(message, options) {
        let embed;
        if (typeof message == "object") {
            embed = message;
            message = undefined;
        }
        let DATA = { content: message, embeds: embed ? [embed] : undefined };
        if (options) {
            DATA = Object.assign(DATA, options);
        }
        return await axios_1.default.post(`https://discord.com/api/v9/webhooks/${this.client.user.id}/${this.token}`, DATA);
    }
    // Get the discord.js channel
    get channel() {
        if (!this.channel_id)
            return undefined;
        let channel = this.guild?.channels.cache.get(this.channel_id);
        if (!channel?.isText())
            return;
        return channel;
    }
    // Get the discord.js guild
    get guild() {
        if (!this.guild_id)
            return undefined;
        const cachedGuild = this.client.guilds.cache.get(this.guild_id);
        if (!cachedGuild) {
            console.log(`Fetching guild: ${this.guild_id}`);
            this.client.guilds.fetch(this.guild_id).then(() => {
                console.log(`Fetched guild: ${this.guild_id}`);
            });
            throw new Error("Fetching guild");
        }
        return cachedGuild;
    }
    // Get the discord.js member
    get member() {
        if (!this.raw_user?.id)
            return undefined;
        if (!this.raw_member)
            return undefined;
        if (!this.guild)
            return undefined;
        return this.guild.members.cache.get(this.raw_user.id) || this.guild.members.add(this.raw_member, true);
    }
    get user() {
        return this.client.users.cache.get(this.raw_user.id) || this.client.users.add(this.raw_user, true);
    }
    // DATABASE STUFF
    async getDbGuild(options) {
        if (!this.guild_id)
            throw Error("No guild id on interaction found");
        return DbUtils.fetch(this.client, this.guild_id, options);
    }
    async updateDbGuild(model) {
        if (!this.guild_id)
            throw Error("No guild id on interaction found");
        await DbUtils.update(this.client, this.guild_id, model);
        return;
    }
}
exports.Interaction = Interaction;
