"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionDataOption = exports.InteractionDataMentions = exports.InteractionData = void 0;
const discord_js_1 = require("discord.js");
class InteractionData {
    constructor(client, data, guild) {
        this.data = data;
        this.id = this.data.id;
        this.name = this.data.name;
        this.resolved = this.data.resolved;
        this.client = client;
        this.guild = guild;
    }
    get options() {
        if (!this.data.options)
            return [];
        else
            return this.data.options.map((option) => new InteractionDataOption(option));
    }
    get mentions() {
        if (!this.data.resolved)
            return undefined;
        return new InteractionDataMentions(this.client, this.data.resolved, this.guild);
    }
}
exports.InteractionData = InteractionData;
class InteractionDataMentions {
    constructor(client, resolve_data, guild) {
        this.data = resolve_data;
        this.client = client;
        this.guild = guild;
    }
    get channels() {
        if (!this.data.channels)
            return undefined;
        let collection = new discord_js_1.Collection();
        Object.values(this.data.channels).map((channel) => collection.set(channel.id, this.client.channels.add(channel)));
        return collection;
    }
    get members() {
        if (!this.data.members)
            return undefined;
        if (!this.guild)
            throw new Error("Resolved member in data, but no guild");
        let collection = new discord_js_1.Collection();
        Object.keys(this.data.members).map((member_id) => collection.set(member_id, this.guild.members.add(this.data.members[member_id])));
        return collection;
    }
    get users() {
        if (!this.data.users)
            return undefined;
        let collection = new discord_js_1.Collection();
        Object.values(this.data.users).map((user) => collection.set(user.id, this.client.users.add(user)));
        return collection;
    }
    get roles() {
        if (!this.data.roles)
            return undefined;
        if (!this.guild)
            throw new Error("Resolved role in data, but no guild");
        let collection = new discord_js_1.Collection();
        Object.values(this.data.roles).map((role) => collection.set(role.id, this.guild.roles.add(role)));
        return collection;
    }
}
exports.InteractionDataMentions = InteractionDataMentions;
class InteractionDataOption {
    constructor(data) {
        this.data = data;
        this.name = this.data.name;
        this.value = this.data.value;
        this.type = this.data.type;
    }
    get options() {
        if (!this.data.options)
            return [];
        else
            return this.data.options.map((option) => new InteractionDataOption(option));
    }
}
exports.InteractionDataOption = InteractionDataOption;
